import base64
from bs4 import BeautifulSoup
import datetime
from playwright.async_api import async_playwright
import asyncio
from .config import Config
config = Config()

_mikan_url = config._mikan_url

class Bangumi:
    def __init__(self, name: str, update_time: datetime.date, url: str):
        self.name = name
        self.update_time = update_time
        self.url = url
        self.poster_url = None
        self.bangumi_link = None  
        self.rating_score = None  
        self.rating_description = None  
        self.shoot = None 

    async def fetch_bangumi_info(self, page):
        """从番剧的 URL 中提取 Bangumi 番组计划链接，并获取评分信息"""
        if not self.url:
            print(f"No URL found for {self.name}")
            return

        # 使用 Playwright 访问番剧详情页
        await page.goto(self.url)
        content = await page.content()
        soup = BeautifulSoup(content, "html.parser")

        # 提取 Bangumi 番组计划链接
        bangumi_info_list = soup.find_all("p", class_="bangumi-info")
        for bangumi_info in bangumi_info_list:
            if "Bangumi番组计划链接" in bangumi_info.text:
                bangumi_link_tag = bangumi_info.find("a", class_="w-other-c")
                if bangumi_link_tag:
                    self.bangumi_link = bangumi_link_tag["href"]

        # 访问 Bangumi 页面并提取评分信息
        if self.bangumi_link:
            await page.goto(self.bangumi_link)
            await page.wait_for_timeout(5000)  # 等待页面加载

            # 提取评分信息
            rating_div = await page.query_selector("div[rel='v:rating']")
            if rating_div:
                rating_score = await rating_div.query_selector("span.number")
                rating_description = await rating_div.query_selector("span.description")
                self.rating_score = await rating_score.inner_text()
                self.rating_description = await rating_description.inner_text()
            else:
                print(f"No rating found for {self.name}")
            target_div = page.locator('div.subject_tag_section')
            await target_div.wait_for(state='visible')
            div = await target_div.screenshot()
            self.shoot = base64.b64encode(div).decode()

class HomePage:
    """表示 Mikanani 主页的类"""
    def __init__(self):
        self.periods = []
        self.period = ""
        self._reset_bangumi_data()

    def _reset_bangumi_data(self):
        """重置番剧数据"""
        self.mon = []
        self.tue = []
        self.wed = []
        self.thu = []
        self.fri = []
        self.sat = []
        self.sun = []

    def _find_target(self, weekday: str):
        """根据星期几返回对应的番剧列表"""
        if weekday == "星期一":
            return self.mon
        elif weekday == "星期二":
            return self.tue
        elif weekday == "星期三":
            return self.wed
        elif weekday == "星期四":
            return self.thu
        elif weekday == "星期五":
            return self.fri
        elif weekday == "星期六":
            return self.sat
        elif weekday == "星期日":
            return self.sun
        else:
            return None

    def feed_p(self, soup: BeautifulSoup):
        """从 BeautifulSoup 对象中提取番剧数据"""
        subsoups = soup.find_all("div", "m-home-week-item")  # 找到所有番剧容器
        for subsoup in subsoups:
            weekday = subsoup.find("span", "monday").text.strip()  # 提取星期几
            target = self._find_target(weekday)  # 获取目标列表
            if target is None:
                continue  # 跳过非星期几的番剧

            items = subsoup.find_all("div", "m-week-square")  # 找到所有番剧信息
            for item in items:
                a_tag = item.find("a", title=True)  # 找到包含标题的 <a> 标签
                if a_tag is None:
                    continue  # 如果 a_tag 不存在，跳过该番剧

                title = a_tag["title"]  # 提取标题
                img_tag = a_tag.find("img")  # 找到图片标签
                img_src = img_tag["data-src"] if img_tag and "data-src" in img_tag.attrs else None  # 提取图片链接

                # 创建 Bangumi 对象并添加到目标列表
                bangumi = Bangumi(
                    name=title,
                    update_time=datetime.date.today(),  # 如果没有日期信息，使用当前日期
                    url=_mikan_url + a_tag["href"] if a_tag["href"] else None  # 拼接完整 URL
                )
                bangumi.poster_url = _mikan_url+img_src  # 添加图片链接
                target.append(bangumi)

    def feed(self, soup: BeautifulSoup):
        psoup = soup.find("li", "sk-col dropdown date-btn")
        if psoup is not None:
            self.period = psoup.find_next("div", "sk-col date-text").contents[0].strip()
            periods = psoup.find_all(lambda tag: tag.name == "a" and tag.has_attr("data-season"))
            for p in periods:
                self.periods.append("{0} {1}".format(p["data-year"], p.contents[0].strip()))
        self._reset_bangumi_data()  # 重置番剧数据
        self.feed_p(soup)  # 提取番剧数据

async def get_homepage(page):
    await page.goto(_mikan_url)
    content = await page.content()
    soup = BeautifulSoup(content, "html.parser")
    hp = HomePage()
    hp.feed(soup)
    return hp


import base64
from nonebot import on_command
from nonebot.plugin import PluginMetadata 
from nonebot.adapters import Message
from nonebot.typing import T_State
from nonebot.adapters.onebot.v11 import (
    Message,
    MessageSegment
)
from playwright.async_api import async_playwright
from PIL import Image
from io import BytesIO
import httpx
from .config import Config
from .anime import get_homepage


__plugin_meta__ = PluginMetadata(
    name="追番小工具",
    description="通过mikan project进行每日番剧推送： /追番 ",
    usage="发送 /追番 进行指令推送",
    type="application",
    homepage="https://github.com/lbsucceed/nonebot-plugin-track-anime",
    config=Config,
    supported_adapters={"~onebot.v11"}
)


track = on_command("追番",aliases={"track"})

@track.got("weekday", prompt="请输入需要查询的日期（例：星期一）：")
async def _(state: T_State):
    async with async_playwright() as p:
        browser = await p.chromium.launch()
        page = await browser.new_page()

        homepage = await get_homepage(page)
        weekday = str(state["weekday"])
        if weekday not in ["星期一", "星期二", "星期三", "星期四", "星期五", "星期六", "星期日"]:
            await track.reject('错误，请重新输入正确的日期')
        target_list = homepage._find_target(weekday)
        message = ""
        for idx, bangumi in enumerate(target_list, start=1):
            message += f"{idx}. {bangumi.name}\n"  
        await track.send(Message(message))
        state["page"] = page
        state["target_list"] = target_list
        state["browser"] = browser  

@track.got("number", prompt="输入所需要查看番剧的序号")
async def _(state: T_State):
    num = int(str(state["number"]))
    target_list = state["target_list"]

    async with async_playwright() as p:
        browser = await p.chromium.launch()
        page = await browser.new_page()

        if 1 <= num <= len(target_list):
            selected_bangumi = target_list[num - 1]
            name = selected_bangumi.name
            await selected_bangumi.fetch_bangumi_info(page)  
            url = selected_bangumi.poster_url
            async with httpx.AsyncClient() as client:
                response = await client.get(url)
                image = Image.open(BytesIO(response.content))
                with BytesIO() as buffer:
                    image.save(buffer, format="PNG")
                    image_base64 = base64.b64encode(buffer.getvalue()).decode("utf-8")

            link = selected_bangumi.bangumi_link
            description = f"{selected_bangumi.rating_score} ({selected_bangumi.rating_description})"
            shoot = selected_bangumi.shoot
            await track.finish(
                f"{link}"+"\n"
                +f"{name}"+"\n"
                + f"{description}"+"\n"
                + (("-" * 30))+"\n"
                + MessageSegment.image(f"base64://{image_base64}")+"\n"
                + (("-" * 30))+"\n"
                + MessageSegment.image(f"base64://{shoot}")
            )

            await page.close()
            await browser.close()
        else:
            await track.reject('输入的序号无效,请重新输入')




        



